package internal

import (
	"context"
	"encoding/json"
	"fmt"
	"os"
	"time"

	"github.com/minio/minio-go/v7"
	"github.com/nats-io/nats.go"
)

func Process(ctx context.Context, mc *minio.Client, nc *nats.Conn, dryRun bool) {
	_, err := nc.Subscribe("minioevents", func(m *nats.Msg) {
		// fmt.Printf("Received a message: %s\n", string(m.Data))
		var event Event
		if err := json.Unmarshal(m.Data, &event); err != nil {
			fmt.Println("wrong event format")
			return
		}
		bucket, objectKey := event.Bucket(), event.ObjectKey()
		if bucket == "" || objectKey == "" {
			return
		}

		fmt.Printf("[%s/%s] cleanning triggered by %s\n", bucket, objectKey, event.Name)

		now := time.Now()
		versions := mc.ListObjects(ctx, bucket, minio.ListObjectsOptions{
			Prefix:       objectKey,
			WithVersions: true,
			WithMetadata: true,
		})

		var lastKept *minio.ObjectInfo
		var total, removed int
		for version := range versions {
			total++
			v := version
			if v.IsLatest || lastKept == nil {
				lastKept = &v
				continue
			}
			nowDiff := version.LastModified.Sub(now).Abs()
			checkPrev := func(condition bool) {
				if condition {
					if !dryRun {
						err := mc.RemoveObject(ctx, bucket, version.Key, minio.RemoveObjectOptions{
							VersionID: version.VersionID,
						})
						if err != nil {
							fmt.Println(err)
							return
						}
					}
					removed++
					fmt.Printf("[%s/%s] removed version %s from %s (%d/%d)\n", bucket, objectKey, version.VersionID, version.LastModified, removed, total)
				} else {
					lastKept = &v
				}
			}
			switch {
			case nowDiff.Hours() < 1:
				checkPrev(lastKept.LastModified.Year() == version.LastModified.Year() && lastKept.LastModified.YearDay() == version.LastModified.YearDay() && lastKept.LastModified.Hour() == version.LastModified.Hour() && lastKept.LastModified.Minute() == version.LastModified.Minute())
			case nowDiff.Hours() < 120:
				checkPrev(lastKept.LastModified.Year() == version.LastModified.Year() && lastKept.LastModified.YearDay() == version.LastModified.YearDay() && lastKept.LastModified.Hour() == version.LastModified.Hour())
			case nowDiff.Hours() < 720:
				checkPrev(lastKept.LastModified.Year() == version.LastModified.Year() && lastKept.LastModified.YearDay() == version.LastModified.YearDay())
			default:
				_, w1 := lastKept.LastModified.ISOWeek()
				_, w2 := version.LastModified.ISOWeek()
				checkPrev(lastKept.LastModified.Year() == version.LastModified.Year() && w1 == w2)
			}

		}
		fmt.Printf("[%s/%s] finished in %v\n", bucket, objectKey, time.Since(now))
	})
	if err != nil {
		fmt.Println(err)
		os.Exit(1)
	}

	<-ctx.Done()

	fmt.Println("terminated")
}
